#!/bin/bash -x
set -e
if [  $UID != 0 ]
then
	zenity --error --title "Creador de pendrive" --text="Es necesario ejecutarlo como super usuario"
	exit 20
fi

if [ $(basename `which pv`) != "pv" ]; then
	zenity --error --title "Creador de pendrive" --text="Es necesario el comando 'pv'"
	exit 5
fi
buscapincho(){
	# Buscamos los posibles pendrive, para ello usamos udisks con varias opciones.
	for pen in $(udisks --enumerate-device-files|grep -v "/dev/disk")
		#aqui le pedimos a udisks la lista de dispositivos posibles
	do

		udisks  --show-info $pen|
		tr -s ' ' ' '|
		grep -C70 "interface: usb"|
		grep -C70 "removable: 1"|
		grep -C70 "has media: 1"|
		grep device-file|
		awk '{print $2}'
		#aqui comprobamos cuales de los dispositivos encontrados son pendrive, tambien evitamos que los lectores de sd sean considerados como dispositivos validos.
	done
}
desmonta() {
	# Desmontamos todas las particiones de un dispositivo concreto.
	for i in $(mount|grep $1|awk '{print $1}' )
	do
		umount $i
	done
}
if [ "$(buscapincho)" = "" ]
then
	zenity --error --title "Creador de pendrive" --text="Tiene que introducir un pendrive"
	exit 10
elif $(zenity --question --title "Creador de pendrive" --text="Advertencia:\nEl uso de esta aplicación implica el borrado del llavero usb utilizado.\n¿Desea continuar?")
then
	# Elegimos el pendrive, para ello mostramos una lista de pendrive.
	PINCHO=$(buscapincho|zenity --list --column unidades --title="Creador de pendrive" --text "¿Qué pendrive usar?")
	if [ "$?" == 1 ] ; then exit 10; fi
	# seleccionamos la imagen a usar.
	IMAGEN=$(zenity --file-selection --title="Selecciona la imagen del pendrive")
	if [ "$?" == 1 ] ; then exit 15; fi
else
	exit 123
fi
if [ $(fdisk -l $PINCHO|grep bytes|grep Disco|sed "s/^\ //g"|awk '{print $5}') -le $(du $IMAGEN|sed "s/^\ //g"|awk '{print $1}') ]
then
        zenity --error --title "Creador de pendrive" --text="Es necesario un pendrive de mayor tamaño"
        exit 129
fi
## AQUI COMENZAMOS LA CREACION DEL PENDRIVE.
desmonta $PINCHO # desmontamos el pendrive si se automonta.
LOOP=$(losetup -f) # buscamos un dispositivo loop libre.
if $(file $IMAGEN|grep -q "ISO 9660 CD-ROM")
then
	# Opciones para montar una imagen iso.
	losetup $LOOP $IMAGEN
elif $(file $IMAGEN|grep -q "x86 boot sector")
then
	losetup $LOOP $IMAGEN # Configuramos el dispositivo loop para que use la imagen que elegimos.
	# Buscamos el inicio de la aprticion. La idea es hacer un loop en el que variamos su inicio (offset).
	EPART=$(sfdisk -d $LOOP |
	        grep -e "Id=cd" -e "Id= b" -e "Id= c"|
		sed s/\ //g|
		tr -s '=' ' '|
		awk '{print $3}'|
		cut -d',' -f1|
		head -n1) 
	TSECTOR=$(fdisk -l $LOOP |
		  grep bytes|
		  awk '{print $7}'|
		  tail -n1)
	losetup -d $LOOP # Desvinculado el dispositivo loop de la imagen.
	LOOP=$(losetup -f)
	losetup -o $(( $EPART * $TSECTOR )) $LOOP $IMAGEN #lo volvemos a vincular teniendo en cuenta el offset.
else    
     	zenity --error --title "Creador de pendrive" --text="Imagen de pendrive no valida"
	exit 142
fi
DEST=`pwd`/.destino
ORIG=`pwd`/.origen
if ! [ -e $DEST ]; then mkdir $DEST; fi
if ! [ -e $ORIG ]; then mkdir $ORIG; fi
mount $LOOP $ORIG # montamos el dispositivo loop.
# eliminamos cualquier rastro de sistema de archivos, tabla de particiones del dispositivo dado.
(dd if=/dev/zero of=$PINCHO bs=1024 count=1024;fdisk $PINCHO << EOF
n
p
1


t
c
a
1
w
EOF
mkdosfs -F 32 ${PINCHO}1
mount ${PINCHO}1 $DEST)|zenity --progress --title "Creador de pendrive" --text "iniciando pendrive" --pulsate --auto-close
rsync --exclude "repositorio.iso" -rP $ORIG/* $DEST|
while read i
do 
	cosa=$(echo $i|
	       grep to-check|
	       tr -s ' ' ' '|
	       cut -d'=' -f2|
	       cut -d')' -f1)
	total=$(echo $cosa|
	        cut -d'/' -f2)
	parcial=$(echo $cosa|
		  cut -d'/' -f1)
	if [ $parcial ] && [ $total ]
	then 
		echo $(( ( $total - $parcial ) * 100 / $total ))
	fi
done|zenity --progress --title "Creador de pendrive" --text "Copiando componentes iniciales" --percentage=0 --auto-close
(pv -n $ORIG/repositorio.iso > $DEST/repositorio.iso) 2>&1 | zenity --progress --title "Creador de pendrive" --text "Completando proceso de copia" --percentage=0 --auto-close
(echo "finalizando"
sync
umount $ORIG
rmdir $ORIG
losetup -d $LOOP
sync
grub-install --root-directory=$DEST $PINCHO
umount $DEST
rmdir $DEST
sync)|zenity --progress --title "Finalizando pendrive" --text "Espere por favor" --pulsate --auto-close
zenity --info --title "Creador de pendrive" --text "Proceso finalizado.\nDesconecte el pendrive."
exit 0
